
-- Collection of LUA utility functions
-- part of smplayer2
-- assembled by A. Schiffler

-- -------------------

-- This function returns a deep copy of a given table. 

-- The function also copies the metatable to the new table if there is one, 
-- so the behaviour of the copied table is the same as the original.
function deepcopy(object)
 local lookup_table = {}
 local function _copy(object)
  if type(object) ~= "table" then
   return object
  elseif lookup_table[object] then
   return lookup_table[object]
  end
  local new_table = {}
  lookup_table[object] = new_table
  for index, value in pairs(object) do
    new_table[_copy(index)] = _copy(value)
  end
  return setmetatable(new_table, getmetatable(object))
 end
 return _copy(object)
end

-- -------------------

-- PHP style string explode

-- Split a string into an array at a divider text
-- Example: explode(" and ","one and two and three and four") --> {"one","two","three","four"}
-- Compatibility: Lua 5.0 and 5.1, code by Richard Warburton
function explode(div,str)
 if (div=='') then return false end
 local pos,arr = 0,{}
 -- for each divider found
 for st,sp in function() return string.find(str,div,pos,true) end do
  table.insert(arr,string.sub(str,pos,st-1)) -- Attach chars left of current divider
  pos = sp + 1 -- Jump past current divider
 end
 table.insert(arr,string.sub(str,pos)) -- Attach chars right of last divider
 return arr
end

-- -------------------

-- Find the length of a file
--   filename: file name
-- returns
--   len: length of file
--   asserts on error
function length_of_file(filename)
  local fh = assert(io.open(filename, "rb"))
  local len = assert(fh:seek("end"))
  fh:close()
  return len
end

-- Return true if file exists and is readable.
function file_exists(path)
  local file = io.open(path, "rb")
  if file then file:close() end
  return file ~= nil
end

-- -------------------

-- Simple save-array-to-file function

-- Example: 
--  local arrayFile = "Array.txt"
--  local array = { 1, 5, 12.3, 0.25, -45.01, 7 }
--  save_array(array,arrayFile);
-- And read back the values with: 
--  local array = dofile(arrayFile)

function save_array(array, arrayFile)
 local f = io.open(arrayFile, "w")
 f:write("return {\n")
 for i, v in ipairs(array) do
  f:write("[" .. i .. "]=" .. v .. ",\n") 
 end
 f:write("}\n")
 f:close()
end

-- -------------------

-- table serialization utility
-- by Steve Dekorte, http://www.dekorte.com, Apr 2000, Freeware

--pickle(table)
function pickle(t)
 return Pickle:clone():pickle_(t)
end

Pickle = {
 clone = function (t) local nt={}; for i, v in t do nt[i]=v end return nt end 
}

function Pickle:pickle_(root)
 if type(root) ~= "table" then 
  error("Can only pickle tables, not ".. type(root).."s")
 end
 self._tableToRef = {}
 self._refToTable = {}
 local savecount = 0
 self:ref_(root)
 local s = ""
 while table.getn(self._refToTable) > savecount do
  savecount = savecount + 1
  local t = self._refToTable[savecount]
  s = s.."{"
  for i, v in t do
   s = string.format("%s[%s]=%s,", s, self:value_(i), self:value_(v))
  end
  s = s.."},\n"
 end
 return string.format("{%s}", s)
end

function Pickle:value_(v)
 local vtype = type(v)
 if     vtype == "string" then return string.format("%q", v)
 elseif vtype == "number" then return v
 elseif vtype == "table" then return "{"..self:ref_(v).."}"
 else --error("pickle a "..type(v).." is not supported")
 end  
end

function Pickle:ref_(t)
 local ref = self._tableToRef[t]
 if not ref then 
 if t == self then error("can't pickle the pickle class") end
  table.insert(self._refToTable, t)
  ref = table.getn(self._refToTable)
  self._tableToRef[t] = ref
 end
 return ref
end

-- unpickle(string)
function unpickle(s)
 if type(s) ~= "string" then
  error("can't unpickle a "..type(s)..", only strings")
 end
 local gentables = loadstring("return "..s)
 local tables = gentables()      
 for tnum = 1, table.getn(tables) do
  local t = tables[tnum]
  local tcopy = {}; for i, v in t do tcopy[i] = v end
  for i, v in tcopy do
   local ni, nv
   if type(i) == "table" then ni = tables[i[1]] else ni = i end
   if type(v) == "table" then nv = tables[v[1]] else nv = v end
   t[ni] = nv
  end
 end
 return tables[1]
end

-- -------------------

-- round number to certain  number of decimal point
function round(num, idp)
 local mult = 10^(idp or 0)
 return math.floor(num * mult + 0.5) / mult
end

-- -------------------

-- Concat the contents of the parameter list,
-- separated by the string delimiter (just like in perl)
-- Example: strjoin(", ", {"Anna", "Bob", "Charlie", "Dolores"})
function strjoin(delimiter, list)
 local len = getn(list)
 if len == 0 then 
  return "" 
 end
 local string = list[1]
 for i = 2, len do 
  string = string .. delimiter .. list[i] 
 end
 return string
end

-- Split text into a list consisting of the strings in text,
-- separated by strings matching delimiter (which may be a pattern). 
-- Example: strsplit(",%s*", "Anna, Bob, Charlie,Dolores")
function strsplit(delimiter, text)
 local list = {}
 local pos = 1
 if strfind("", delimiter, 1) then
  tinsert(list, text)
  return list
 end
 while 1 do
  local first, last = strfind(text, delimiter, pos)
  if first then
   tinsert(list, strsub(text, pos, first-1))
   pos = last+1
  else
   tinsert(list, strsub(text, pos))
   break
  end
 end
 return list
end

-- -------------------

-- timezone functions

-- Compute the difference in seconds between local time and UTC.
-- Example: timezone = get_timezone()
function get_timezone()
  local now = os.time()
  return os.difftime(now, os.time(os.date("!*t", now)))
end

-- Return a timezone string in ISO 8601:2000 standard form (+hhmm or -hhmm)
-- Example: tzoffset = get_tzoffset(timezone)
function get_tzoffset(timezone)
  local h, m = math.modf(timezone / 3600)
  return string.format("%+.4d", 100 * h + 60 * m)
end

-- -------------------
